//  How to run @NSC:
//      module load CUDA/10.1.243-GCC-8.3.0
//      nvcc -o devinfo -I . devinfo.c
//      srun --reservation=fri --gpus=1 ./devinfo

#include <stdio.h>
#include <cuda.h>
#include <cuda_runtime.h>
#include "helper_cuda.h"

int main(int argc, char **argv) 
{
 
  // Get number of GPUs
  int deviceCount = 0;
  cudaError_t error = cudaGetDeviceCount(&deviceCount);

  if (error != cudaSuccess) 
  {
    printf("cudaGetDeviceCount error %d\n-> %s\n", error, cudaGetErrorString(error));
    exit(EXIT_FAILURE);
  }


  // Get propreties of each device
  for (int dev = 0; dev < deviceCount; dev++) 
  {

    struct cudaDeviceProp prop;
    int value;

    cudaGetDeviceProperties(&prop, dev);
    printf("\n======= Device %d: \"%s\" =======\n", dev, prop.name);

    printf("\ncudaDeviceGetProperties:\n");
    printf("  GPU Clock Rate (MHz):                          %d\n", prop.clockRate/1000);
    printf("  Memory Clock Rate (MHz):                       %d\n", prop.memoryClockRate/1000);
    printf("  Memory Bus Width (bits):                       %d\n", prop.memoryBusWidth);
    printf("  Peak Memory Bandwidth (GB/s):                  %.2f\n", 2.0*prop.memoryClockRate*(prop.memoryBusWidth/8)/1.0e6);
    printf("  CUDA Cores/MP:                                 %d\n", _ConvertSMVer2Cores(prop.major, prop.minor));
    printf("  CUDA Cores:                                    %d\n", _ConvertSMVer2Cores(prop.major, prop.minor) * prop.multiProcessorCount);
    printf("  Total amount of global memory:                 %.0f GB\n", prop.totalGlobalMem / 1073741824.0f);
    printf("  Total amount of shared memory per block:       %zu kB\n", prop.sharedMemPerBlock/1024);
    printf("  Total number of registers available per block: %d\n", prop.regsPerBlock);
    printf("  Maximum number of threads per block:           %d\n", prop.maxThreadsPerBlock);
    printf("  Warp size:                                     %d\n", prop.warpSize);
    printf("  Maximum number of threads per MP:              %d\n", prop.maxThreadsPerMultiProcessor);
    printf("  Maximum number of registers per MP:            %d\n", prop.regsPerMultiprocessor);
    printf("  Total amount of shared memory per MP:          %d\n", prop.sharedMemPerMultiprocessor/1024);
    printf("  Max dimension size of a thread block (x,y,z): (%d, %d, %d)\n", prop.maxThreadsDim[0], prop.maxThreadsDim[1], prop.maxThreadsDim[2]);
    printf("  Max dimension size of a grid size    (x,y,z): (%d, %d, %d)\n", prop.maxGridSize[0], prop.maxGridSize[1], prop.maxGridSize[2]);

    printf("\ncudaDeviceGetAttribute:\n");
    cudaDeviceGetAttribute (&value, cudaDevAttrComputeCapabilityMajor, dev);
    printf("  Major compute capability version number:      %d\n", value);
    cudaDeviceGetAttribute (&value, cudaDevAttrComputeCapabilityMinor, dev);
    printf("  Minor compute capability version number:      %d\n", value);
    cudaDeviceGetAttribute (&value, cudaDevAttrL2CacheSize, dev);
    printf("  Size of L2 cache in bytes:                    %d\n", value);
//    cudaDeviceGetAttribute (&value, cudaDevAttrMaxBlocksPerMultiprocessor, dev);  // works in newer CUDA versions
//    printf("  Max thread blocks that can reside on a SM:    %d\n", value);  
  }
}
